import { FIRST_ARRAY_ELEMENT, SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { getMultiBioEntity } from '@/redux/bioEntity/bioEntity.thunks';
import { openReactionDrawerById } from '@/redux/drawer/drawer.slice';
import { getReactionsByIds } from '@/redux/reactions/reactions.thunks';
import { AppDispatch } from '@/redux/store';
import { searchFitBounds } from '@/services/pluginsManager/map/triggerSearch/searchFitBounds';
import { ElementSearchResult, Reaction } from '@/types/models';
import { PayloadAction } from '@reduxjs/toolkit';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';

/* prettier-ignore */
export const handleReactionResults =
  (dispatch: AppDispatch, closestSearchResult: ElementSearchResult, hasFitBounds?: boolean, fitBoundsZoom?: number) =>
    async ({ id }: ElementSearchResult): Promise<void> => {
      const data = await dispatch(getReactionsByIds([id])) as PayloadAction<Reaction[] | undefined>;
      const payload = data?.payload;
      if (!data || !payload || payload.length === SIZE_OF_EMPTY_ARRAY) {
        return;
      }

      const reaction = payload[FIRST_ARRAY_ELEMENT];
      const { products, reactants, modifiers } = reaction;
      const productsIds = products.map(p => p.aliasId);
      const reactantsIds = reactants.map(r => r.aliasId);
      const modifiersIds = modifiers.map(m => m.aliasId);
      const bioEntitiesIds = [...productsIds, ...reactantsIds, ...modifiersIds].map(identifier => String(identifier));

      dispatch(openReactionDrawerById(reaction.id));
      await dispatch(
        getMultiBioEntity({
          searchQueries: bioEntitiesIds,
          isPerfectMatch: true },
        )
      ).unwrap().then((bioEntityContents) => {
        PluginsEventBus.dispatchEvent('onSearch', {
          type: 'bioEntity',
          searchValues: [closestSearchResult],
          results: [bioEntityContents],
        });

        if (hasFitBounds) {
          searchFitBounds(fitBoundsZoom);
        }
      });
    };
